import datetime

import pytest

from wcs import fields
from wcs.formdef import FormDef
from wcs.qommon.http_request import HTTPRequest
from wcs.testdef import TestDef, TestError

from .utilities import clean_temporary_pub, create_temporary_pub


@pytest.fixture
def pub():
    pub = create_temporary_pub()

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.write_cfg()

    FormDef.wipe()
    return pub


def teardown_module(module):
    clean_temporary_pub()


def test_testdef_slug_generation(pub):
    testdef = TestDef()
    testdef.name = 'test'
    testdef.object_type = 'formdef'
    testdef.object_id = '1'
    testdef.store()
    assert testdef.slug == 'test'

    testdef.slug = testdef.id = None
    testdef.store()
    assert testdef.slug == 'test-2'

    testdef.slug = testdef.id = None
    testdef.object_id = '2'
    testdef.store()
    assert testdef.slug == 'test'


def test_page_post_conditions(pub):
    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.TitleField(id='0', label='Title', type='title'),
        fields.PageField(
            id='1',
            label='1st page',
            type='page',
            post_conditions=[
                {'condition': {'type': 'django', 'value': 'form_var_text == "a"'}, 'error_message': 'Error'}
            ],
        ),
        fields.StringField(id='2', label='Text', varname='text'),
        fields.PageField(
            id='3',
            label='2nd page',
            type='page',
            post_conditions=[
                {'condition': {'type': 'django', 'value': 'form_var_text2 == "a"'}, 'error_message': 'Error'}
            ],
        ),
        fields.StringField(id='4', label='Text', varname='text2'),
    ]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['2'] = 'a'
    formdata.data['4'] = 'a'

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.run(formdef)

    formdata.data['4'] = 'b'
    testdef = TestDef.create_from_formdata(formdef, formdata)
    with pytest.raises(TestError) as excinfo:
        testdef.run(formdef)
    assert str(excinfo.value) == 'Page 2 post condition was not met (form_var_text2 == "a").'

    formdata.data['2'] = 'b'
    testdef = TestDef.create_from_formdata(formdef, formdata)
    with pytest.raises(TestError) as excinfo:
        testdef.run(formdef)
    assert str(excinfo.value) == 'Page 1 post condition was not met (form_var_text == "a").'


def test_page_post_condition_invalid(pub):
    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.PageField(
            id='0',
            label='1st page',
            type='page',
            post_conditions=[{'condition': {'type': 'django', 'value': '{{}'}, 'error_message': 'Error'}],
        ),
    ]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()

    testdef = TestDef.create_from_formdata(formdef, formdata)
    with pytest.raises(TestError) as excinfo:
        testdef.run(formdef)
    assert str(excinfo.value) == 'Failed to evaluate page 1 post condition.'


def test_field_conditions(pub):
    formdef = FormDef()
    formdef.name = 'test title'
    formdef.fields = [
        fields.StringField(id='1', label='Text', varname='text'),
        fields.StringField(
            id='2',
            label='Text with condition',
            varname='text_cond',
            required=False,
            condition={'type': 'django', 'value': 'form_var_text == "a"'},
        ),
    ]
    formdef.store()

    formdata = formdef.data_class()()
    formdata.just_created()
    formdata.receipt_time = datetime.datetime(2021, 1, 1, 0, 0).timetuple()
    formdata.data['1'] = 'a'
    formdata.data['2'] = 'xxx'

    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.run(formdef)

    formdata.data['1'] = 'b'
    testdef = TestDef.create_from_formdata(formdef, formdata)
    with pytest.raises(TestError) as excinfo:
        testdef.run(formdef)
    assert str(excinfo.value) == 'Tried to fill field "Text with condition" on page 1 but it is hidden.'

    formdata.data['2'] = None
    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.run(formdef)

    formdata.data['1'] = 'a'
    testdef = TestDef.create_from_formdata(formdef, formdata)
    testdef.run(formdef)
